#!/usr/bin/env lua

SCENE_HISTORY_LENGTH = 5

------------------------------------------------------------------------------
-- pretty print contents of a table
------------------------------------------------------------------------------
function printTable(name, t)
    if (t == nil) then
        print(name.." is nil")
        return
    end
    for i,j in pairs(t) do
        if (type(j) == "table") then
            printTable(name.."["..tostring(i).."]", j);
        else
            print(name.."."..tostring(i).."='"..tostring(j).."'")
        end
    end
    print(name.." size = "..#name)
end

------------------------------------------------------------------------------
-- pretty print contentens of scene history merge object
------------------------------------------------------------------------------
function printMerge(merge)
    local mrgs = scene.getSources(merge)
    local str = ""

    for i,v in pairs(mrgs) do
        str = str..tostring(v).." "
    end

    print("( "..str..") -> "..tostring(merge.target))
end

------------------------------------------------------------------------------
-- pretty print contentens of scene history split object
------------------------------------------------------------------------------
function printSplit(split)
    local splts = scene.getTargets(split)
    local str = ""

    for i,v in pairs(splts) do
        str = str..tostring(v).." "
    end

    print(tostring(split.source).." -> ( "..str..")")
end


------------------------------------------------------------------------------
-- check if one the object is one of the split targets
-------------------------------------------------------------------------------
function isSplitTarget(split, objectID)
    local trgs = scene.getTargets(split)
    for i,v in pairs(trgs) do
        if (v == objectID) then
            return true
        end
    end

    return false
end

-------------------------------------------------------------------------------
-- follow object merges and splits back in time
--
-- for all objects in the provided table check:

--    if the object have been merged in this frame
--    in the case replace it with merge sources
--
--    if the object is an result of an split in this frame
--    replace it with the split
--
-- @param object_ids an array of the objects to follow
-- @param frame the frame number where to follow the objects
--
-- @return an array of objects or thier merge sources in requested frame
-------------------------------------------------------------------------------
function follow_merge_and_split(object_ids, frame)
    local res = {}

    -- copy objects table
    for i,v in pairs(object_ids) do
        res[i] = true
    end

    -- check the splits in this frame
    local splits = sceneHistory.getSceneObjectSplits(frame)
    for obj_id,v in pairs(object_ids) do
        for j,split in pairs(splits) do
            if (isSplitTarget(split, obj_id)) then
                res[obj_id] = nil
                res[split.source] = true
            end
        end
    end

    -- check the mergers in this frame
    local mergers = sceneHistory.getSceneObjectMergers(frame)    
    for obj_id,obj in pairs(object_ids) do
       for j,merge in pairs(mergers) do
           if (merge.target == obj_id) then
               res[obj_id] = nil             
               local srcs = scene.getSources(merge)
               for k,src in pairs(srcs) do
                     res[src] = true
               end
           end
       end
    end

    return res
end

-------------------------------------------------------------------------------
-- follow object merges back in time
--
-- for all objects in the provided table check
-- if the object have been merged in this frame,
-- in that case replace it with merge sources
--
--
-- @param object_ids an array of the objects to follow
-- @param frame the frame number where to follow the objects
--
-- @return an array of objects or thier merge sources in requested frame
-------------------------------------------------------------------------------
function follow_merges(object_ids, frame)
    local res = {}

    -- copy objects table
    for i,v in pairs(object_ids) do
        res[i] = true
    end

    -- check the mergers in this frame
    local mergers = sceneHistory.getSceneObjectMergers(frame)    
    for obj_id,obj in pairs(object_ids) do
       for j,merge in pairs(mergers) do
           if (merge.target == obj_id) then
               res[obj_id] = nil             
               local srcs = scene.getSources(merge)
               for k,src in pairs(srcs) do
                     res[src] = true
               end
           end
       end
    end

    return res
end


-------------------------------------------------------------------------------
-- Dump all info in scene history for the specified frame
-------------------------------------------------------------------------------
function printScene(frameNum)
    print("-- frame start --")

    -- print active objects
    local activeObjs = sceneHistory.getActiveSceneObjects(frameNum)
    if (#activeObjs > 0) then
        print("active objects")
        local str = ""
        for i,v in pairs(activeObjs) do
            str = str .. tostring(v.id).." "
        end
        print(str)
    end

    -- print mergers
    local mergers = sceneHistory.getSceneObjectMergers(frameNum)
    if (#mergers > 0) then
        print("mergers")
        for i,v in pairs(mergers) do
            printMerge(v)
        end
    end

    -- print splits
    local splits = sceneHistory.getSceneObjectSplits(frameNum)
    if (#splits > 0) then
        print("splits")
        for i,v in pairs(splits) do
            printSplit(v)
        end
    end


    -- print deletes
    local deletes = sceneHistory.getDeletedSceneObjectIds(frameNum)
    if (#deletes > 0) then
        print("deletes")
        local str = ""
        for i,v in pairs(deletes) do
            str = str..tostring(v).." "
        end
        print(str)
    end
    
end
